import { ActionCollectionConfig, ActionCollectionResult, BaseMetadata, MetadataSetter, StateChanges, StateGetter, StateHook, StateSetter, UseHookConfig } from 'react-hooks-global-states';
import React, { PropsWithChildren } from 'react';
import { LocalStorageConfig } from 'GlobalStore.types';
export type ProviderAPI<Value, Metadata> = {
    setMetadata: MetadataSetter<Metadata>;
    setState: StateSetter<Value>;
    getState: StateGetter<Value>;
    getMetadata: () => Metadata;
    actions: Record<string, (...args: any[]) => void>;
};
type Provider<Value, Metadata extends BaseMetadata = BaseMetadata> = React.FC<PropsWithChildren<{
    value?: Value | ((initialValue: Value) => Value);
    ref?: React.MutableRefObject<ProviderAPI<Value, Metadata>>;
}>>;
type Context<Value, PublicStateMutator, Metadata extends BaseMetadata> = (() => StateHook<Value, PublicStateMutator, Metadata>) & {
    /**
     * Allows you to create a selector hooks
     * This hooks only works when contained in the scope of the provider
     */
    createSelectorHook: <RootSelectorResult, RootDerivate = RootSelectorResult extends never ? Value : RootSelectorResult>(mainSelector?: (state: Value) => RootSelectorResult, { isEqualRoot, isEqual, }?: Omit<UseHookConfig<RootDerivate, Value>, 'dependencies'>) => StateHook<RootDerivate, PublicStateMutator, Metadata>;
};
export interface CreateContext {
    createContext<Value>(state: Value): readonly [
        Context<Value, StateSetter<Value>, BaseMetadata>,
        Provider<Context<Value, StateSetter<Value>, BaseMetadata>>
    ];
    createContext<Value, Metadata extends BaseMetadata, ActionsConfig extends ActionCollectionConfig<Value, Metadata> | {} | null = null, StoreAPI = {
        setMetadata: MetadataSetter<Metadata>;
        setState: StateSetter<Value>;
        getState: StateGetter<Value>;
        getMetadata: () => Metadata;
        actions: ActionsConfig extends null ? null : Record<string, (...args: any[]) => void>;
    }>(state: Value, config: Readonly<{
        /**
         * @deprecated We needed to move the actions parameter as a third argument to fix several issues with the type inference of the actions
         */
        actions?: ActionsConfig;
        /**
         * Non reactive information about the state
         */
        metadata?: Metadata;
        /**
         * executes immediately after the store is created
         * */
        onInit?: (args: StoreAPI) => void;
        onStateChanged?: (args: StoreAPI & StateChanges<Value>) => void;
        onSubscribed?: (args: StoreAPI) => void;
        /**
         * callback function called every time the state is about to change and it allows you to prevent the state change
         */
        computePreventStateChange?: (args: StoreAPI & StateChanges<Value>) => boolean;
    } & LocalStorageConfig>): readonly [
        Context<Value, ActionsConfig extends null ? StateSetter<Value> : ActionCollectionResult<Value, Metadata, ActionsConfig>, Metadata>,
        Provider<Context<Value, ActionsConfig extends null ? StateSetter<Value> : ActionCollectionResult<Value, Metadata, ActionsConfig>, Metadata>, Metadata>
    ];
    createContext<Value, Metadata extends BaseMetadata, ActionsConfig extends ActionCollectionConfig<Value, Metadata>, StoreAPI = {
        setMetadata: MetadataSetter<Metadata>;
        setState: StateSetter<Value>;
        getState: StateGetter<Value>;
        getMetadata: () => Metadata;
        actions: Record<string, (...args: any[]) => void>;
    }>(state: Value, config: Readonly<{
        /**
         * Non reactive information about the state
         */
        metadata?: Metadata;
        /**
         * executes immediately after the store is created
         * */
        onInit?: (args: StoreAPI) => void;
        onStateChanged?: (args: StoreAPI & StateChanges<Value>) => void;
        onSubscribed?: (args: StoreAPI) => void;
        /**
         * callback function called every time the state is about to change and it allows you to prevent the state change
         */
        computePreventStateChange?: (args: StoreAPI & StateChanges<Value>) => boolean;
    } & LocalStorageConfig>, actions: ActionsConfig): readonly [
        Context<Value, ActionCollectionResult<Value, Metadata, ActionsConfig>, Metadata>,
        Provider<Context<Value, ActionCollectionResult<Value, Metadata, ActionsConfig>, Metadata>>
    ];
    createContext<Value, Metadata extends BaseMetadata, ActionsConfig extends ActionCollectionConfig<Value, Metadata>, StoreAPI = {
        setMetadata: MetadataSetter<Metadata>;
        setState: StateSetter<Value>;
        getState: StateGetter<Value>;
        getMetadata: () => Metadata;
    }>(state: Value, builder: () => ActionsConfig, config?: Readonly<{
        /**
         * Non reactive information about the state
         */
        metadata?: Metadata;
        /**
         * executes immediately after the store is created
         * */
        onInit?: (args: StoreAPI) => void;
        onStateChanged?: (args: StoreAPI & StateChanges<Value>) => void;
        onSubscribed?: (args: StoreAPI) => void;
        /**
         * callback function called every time the state is about to change and it allows you to prevent the state change
         */
        computePreventStateChange?: (args: StoreAPI & StateChanges<Value>) => boolean;
    } & LocalStorageConfig>): readonly [
        Context<Value, ActionCollectionResult<Value, Metadata, ActionsConfig>, Metadata>,
        Provider<Context<Value, ActionCollectionResult<Value, Metadata, ActionsConfig>, Metadata>>
    ];
}
export declare const createContext: {
    <Value>(state: Value): readonly [Context<Value, StateSetter<Value>, BaseMetadata>, Provider<Context<Value, StateSetter<Value>, BaseMetadata>, BaseMetadata>];
    <Value_1, Metadata extends BaseMetadata, ActionsConfig extends {} | ActionCollectionConfig<Value_1, Metadata> = null, StoreAPI = {
        setMetadata: MetadataSetter<Metadata>;
        setState: StateSetter<Value_1>;
        getState: StateGetter<Value_1>;
        getMetadata: () => Metadata;
        actions: ActionsConfig extends null ? null : Record<string, (...args: any[]) => void>;
    }>(state: Value_1, config: Readonly<{
        /**
         * @deprecated We needed to move the actions parameter as a third argument to fix several issues with the type inference of the actions
         */
        actions?: ActionsConfig;
        /**
         * Non reactive information about the state
         */
        metadata?: Metadata;
        /**
         * executes immediately after the store is created
         * */
        onInit?: (args: StoreAPI) => void;
        onStateChanged?: (args: StoreAPI & StateChanges<Value_1>) => void;
        onSubscribed?: (args: StoreAPI) => void;
        /**
         * callback function called every time the state is about to change and it allows you to prevent the state change
         */
        computePreventStateChange?: (args: StoreAPI & StateChanges<Value_1>) => boolean;
    } & LocalStorageConfig>): readonly [Context<Value_1, ActionsConfig extends null ? StateSetter<Value_1> : ActionCollectionResult<Value_1, Metadata, ActionsConfig>, Metadata>, Provider<Context<Value_1, ActionsConfig extends null ? StateSetter<Value_1> : ActionCollectionResult<Value_1, Metadata, ActionsConfig>, Metadata>, Metadata>];
    <Value_2, Metadata_1 extends BaseMetadata, ActionsConfig_1 extends ActionCollectionConfig<Value_2, Metadata_1>, StoreAPI_1 = {
        setMetadata: MetadataSetter<Metadata_1>;
        setState: StateSetter<Value_2>;
        getState: StateGetter<Value_2>;
        getMetadata: () => Metadata_1;
        actions: Record<string, (...args: any[]) => void>;
    }>(state: Value_2, config: Readonly<{
        /**
         * Non reactive information about the state
         */
        metadata?: Metadata_1;
        /**
         * executes immediately after the store is created
         * */
        onInit?: (args: StoreAPI_1) => void;
        onStateChanged?: (args: StoreAPI_1 & StateChanges<Value_2>) => void;
        onSubscribed?: (args: StoreAPI_1) => void;
        /**
         * callback function called every time the state is about to change and it allows you to prevent the state change
         */
        computePreventStateChange?: (args: StoreAPI_1 & StateChanges<Value_2>) => boolean;
    } & LocalStorageConfig>, actions: ActionsConfig_1): readonly [Context<Value_2, ActionCollectionResult<Value_2, Metadata_1, ActionsConfig_1>, Metadata_1>, Provider<Context<Value_2, ActionCollectionResult<Value_2, Metadata_1, ActionsConfig_1>, Metadata_1>, BaseMetadata>];
    <Value_3, Metadata_2 extends BaseMetadata, ActionsConfig_2 extends ActionCollectionConfig<Value_3, Metadata_2>, StoreAPI_2 = {
        setMetadata: MetadataSetter<Metadata_2>;
        setState: StateSetter<Value_3>;
        getState: StateGetter<Value_3>;
        getMetadata: () => Metadata_2;
    }>(state: Value_3, builder: () => ActionsConfig_2, config?: Readonly<{
        /**
         * Non reactive information about the state
         */
        metadata?: Metadata_2;
        /**
         * executes immediately after the store is created
         * */
        onInit?: (args: StoreAPI_2) => void;
        onStateChanged?: (args: StoreAPI_2 & StateChanges<Value_3>) => void;
        onSubscribed?: (args: StoreAPI_2) => void;
        /**
         * callback function called every time the state is about to change and it allows you to prevent the state change
         */
        computePreventStateChange?: (args: StoreAPI_2 & StateChanges<Value_3>) => boolean;
    } & LocalStorageConfig>): readonly [Context<Value_3, ActionCollectionResult<Value_3, Metadata_2, ActionsConfig_2>, Metadata_2>, Provider<Context<Value_3, ActionCollectionResult<Value_3, Metadata_2, ActionsConfig_2>, Metadata_2>, BaseMetadata>];
};
export {};
