import { LocalStorageConfig } from './GlobalStore.types';
import { ActionCollectionConfig, ActionCollectionResult, BaseMetadata, CustomGlobalHookBuilderParams, MetadataSetter, StateChanges, StateGetter, StateHook, StateSetter, StoreTools } from 'react-hooks-global-states';
/**
 * Creates a global hook that can be used to access the state and actions across the application
 * @returns {} - () => [TState, Setter, TMetadata] the hook that can be used to access the state and the setter of the state
 */
export declare const createGlobalState: {
    <State>(state: State): StateHook<State, StateSetter<State>, BaseMetadata>;
    <State_1, Metadata extends BaseMetadata, ActionsConfig extends {} | ActionCollectionConfig<State_1, Metadata> = null, StoreAPI = {
        setMetadata: MetadataSetter<Metadata>;
        setState: StateSetter<State_1>;
        getState: StateGetter<State_1>;
        getMetadata: () => Metadata;
        actions: ActionsConfig extends null ? null : Record<string, (...args: any[]) => void>;
    }>(state: State_1, config: Readonly<{
        /**
         * @deprecated We needed to move the actions parameter as a third argument to fix several issues with the type inference of the actions
         */
        actions?: ActionsConfig;
        /**
         * Non reactive information about the state
         */
        metadata?: Metadata;
        /**
         * executes immediately after the store is created
         * */
        onInit?: (args: StoreAPI) => void;
        onStateChanged?: (args: StoreAPI & StateChanges<State_1>) => void;
        onSubscribed?: (args: StoreAPI) => void;
        /**
         * callback function called every time the state is about to change and it allows you to prevent the state change
         */
        computePreventStateChange?: (args: StoreAPI & StateChanges<State_1>) => boolean;
    } & LocalStorageConfig>): StateHook<State_1, ActionsConfig extends null ? StateSetter<State_1> : ActionCollectionResult<State_1, Metadata, ActionsConfig>, Metadata>;
    <State_2, Metadata_1 extends BaseMetadata, ActionsConfig_1 extends ActionCollectionConfig<State_2, Metadata_1>, StoreAPI_1 = {
        setMetadata: MetadataSetter<Metadata_1>;
        setState: StateSetter<State_2>;
        getState: StateGetter<State_2>;
        getMetadata: () => Metadata_1;
        actions: Record<string, (...args: any[]) => void>;
    }>(state: State_2, config: Readonly<{
        /**
         * Non reactive information about the state
         */
        metadata?: Metadata_1;
        /**
         * executes immediately after the store is created
         * */
        onInit?: (args: StoreAPI_1) => void;
        onStateChanged?: (args: StoreAPI_1 & StateChanges<State_2>) => void;
        onSubscribed?: (args: StoreAPI_1) => void;
        /**
         * callback function called every time the state is about to change and it allows you to prevent the state change
         */
        computePreventStateChange?: (args: StoreAPI_1 & StateChanges<State_2>) => boolean;
    } & LocalStorageConfig>, actions: ActionsConfig_1): StateHook<State_2, ActionCollectionResult<State_2, Metadata_1, ActionsConfig_1>, Metadata_1>;
    <State_3, Metadata_2 extends BaseMetadata, ActionsConfig_2 extends ActionCollectionConfig<State_3, Metadata_2>, StoreAPI_2 = {
        setMetadata: MetadataSetter<Metadata_2>;
        setState: StateSetter<State_3>;
        getState: StateGetter<State_3>;
        getMetadata: () => Metadata_2;
    }>(state: State_3, builder: () => ActionsConfig_2, config?: Readonly<{
        /**
         * Non reactive information about the state
         */
        metadata?: Metadata_2;
        /**
         * executes immediately after the store is created
         * */
        onInit?: (args: StoreAPI_2) => void;
        onStateChanged?: (args: StoreAPI_2 & StateChanges<State_3>) => void;
        onSubscribed?: (args: StoreAPI_2) => void;
        /**
         * callback function called every time the state is about to change and it allows you to prevent the state change
         */
        computePreventStateChange?: (args: StoreAPI_2 & StateChanges<State_3>) => boolean;
    } & LocalStorageConfig>): StateHook<State_3, ActionCollectionResult<State_3, Metadata_2, ActionsConfig_2>, Metadata_2>;
};
/**
 * @description
 * Use this function to create a custom global store.
 * You can use this function to create a store with async storage.
 */
export declare const createCustomGlobalState: <InheritMetadata extends BaseMetadata, CustomConfig extends Record<string, any>>({ onInitialize, onChange, }: CustomGlobalHookBuilderParams<InheritMetadata, CustomConfig>) => <State, Metadata extends BaseMetadata = InheritMetadata, ActionsConfig extends Readonly<ActionCollectionConfig<State, Metadata & InheritMetadata>> = null>(state: State, _config?: Readonly<{
    metadata?: Metadata & InheritMetadata;
    readonly actions?: Readonly<ActionsConfig>;
    onInit?: (args: StoreTools<State, Metadata & InheritMetadata>) => void;
    onStateChanged?: (args: StoreTools<State, Metadata & InheritMetadata> & StateChanges<State>) => void;
    onSubscribed?: (parameters: StoreTools<State, Metadata & InheritMetadata>) => void;
    computePreventStateChange?: (args: StoreTools<State, Metadata & InheritMetadata> & StateChanges<State>) => boolean;
    /**
     * @description
     * Type of the configuration object that the custom hook will require or accept
     */
    config?: CustomConfig;
}>) => StateHook<State, Readonly<ActionsConfig> extends infer T ? T extends Readonly<ActionsConfig> ? T extends null ? StateSetter<State> : ActionCollectionResult<State, Metadata & InheritMetadata, T> : never : never, Metadata & InheritMetadata>;
/**
 * @description
 * Use this function to create a custom global store.
 * You can use this function to create a store with async storage.
 * @deprecated
 */
export declare const createCustomGlobalStateWithDecoupledFuncs: <InheritMetadata extends BaseMetadata, CustomConfig extends Record<string, any>>({ onInitialize, onChange, }: CustomGlobalHookBuilderParams<InheritMetadata, CustomConfig>) => <State, Metadata extends BaseMetadata = InheritMetadata, ActionsConfig extends Readonly<ActionCollectionConfig<State, Metadata & InheritMetadata>> = null>(state: State, _config?: Readonly<{
    metadata?: Metadata & InheritMetadata;
    readonly actions?: Readonly<ActionsConfig>;
    onInit?: (args: StoreTools<State, Metadata & InheritMetadata>) => void;
    onStateChanged?: (args: StoreTools<State, Metadata & InheritMetadata> & StateChanges<State>) => void;
    onSubscribed?: (parameters: StoreTools<State, Metadata & InheritMetadata>) => void;
    computePreventStateChange?: (args: StoreTools<State, Metadata & InheritMetadata> & StateChanges<State>) => boolean;
    /**
     * @description
     * Type of the configuration object that the custom hook will require or accept
     */
    config?: CustomConfig;
}>) => StateHook<State, Readonly<ActionsConfig> extends infer T ? T extends Readonly<ActionsConfig> ? T extends null ? StateSetter<State> : ActionCollectionResult<State, Metadata & InheritMetadata, T> : never : never, Metadata & InheritMetadata>;
